@@ -20,7 +20,7 @@ from pymeteobridgedata.data import (
     ObservationDescription,
 )
 from pymeteobridgedata.exceptions import BadRequest
-from pymeteobridgedata.helpers import Calculations, Conversions
+from pymeteobridgedata.conversion import Conversions
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -37,8 +37,6 @@ class MeteobridgeApiClient:
         ip_address: str,
         units: Optional[str] = UNIT_TYPE_METRIC,
         extra_sensors: Optional[int] = 0,
-        extra_soil_sensors: Optional[int] = 0,
-        extra_leaf_sensors: Optional[int] = 0,
         homeassistant: Optional(bool) = True,
         session: Optional[ClientSession] = None,
     ) -> None:
@@ -49,8 +47,6 @@ class MeteobridgeApiClient:
         self.units = units
         self.homeassistant = homeassistant
         self.extra_sensors = extra_sensors
-        self.extra_soil_sensors = extra_soil_sensors
-        self.extra_leaf_sensors = extra_leaf_sensors
 
         if self.units not in VALID_UNIT_TYPES:
             self.units = UNIT_TYPE_METRIC
@@ -60,7 +56,6 @@ class MeteobridgeApiClient:
         self.req = session
 
         self.cnv = Conversions(self.units, self.homeassistant)
-        self.calc = Calculations()
 
         self.base_url = (f"http://{self.username}:{self.password}@{self.ip_address}"
                          "/cgi-bin/template.cgi?template=")
@@ -99,7 +94,7 @@ class MeteobridgeApiClient:
         if self._device_data is None:
             _LOGGER.error("Logger has not been initialized. "
                           "Run initilaize() function first.")
-            return
+            raise BadRequest("Waiting for valid data from Meteobridge.")
 
         data_fields = self._build_endpoint(FIELDS_OBSERVATION)
         endpoint = f"{self.base_url}{data_fields}"
@@ -107,20 +102,27 @@ class MeteobridgeApiClient:
         data = await self._process_request_result(result, FIELDS_OBSERVATION)
 
         if data is not None:
-            visibility = self.calc.visibility(
+            # Calculated Fields
+            visibility = self.cnv.visibility(
                 self._device_data.elevation,
                 data["air_temperature"],
                 data["relative_humidity"],
                 data["dew_point"]
             )
-            feels_like = self.calc.feels_like(
+            wind_chill = self.cnv.wind_chill(data["air_temperature"], data["wind_gust"])
+            feels_like = self.cnv.feels_like(
                 data["air_temperature"],
                 data["relative_humidity"],
-                data["wind_gust"],
-                data["heat_index"],
-                data["wind_chill"]
+                data["wind_gust"]
             )
-            beaufort_data: BeaufortDescription = self.calc.beaufort(data["wind_avg"])
+            beaufort_data: BeaufortDescription = self.cnv.beaufort(data["wind_avg"])
+            wet_bulb = self.cnv.wetbulb(
+                data["air_temperature"],
+                data["relative_humidity"],
+                data["station_pressure"]
+            )
+
+            # Raw Data Fields
             entity_data = ObservationDescription(
                 key=self._device_data.key,
                 utc_time=self.cnv.utc_from_timestamp(data["utc_time"]),
@@ -133,30 +135,33 @@ class MeteobridgeApiClient:
                 wind_avg=self.cnv.windspeed(data["wind_avg"]),
                 wind_gust=self.cnv.windspeed(data["wind_gust"]),
                 wind_direction=data["wind_direction"],
-                wind_cardinal=self.calc.wind_direction(data["wind_direction"]),
+                wind_cardinal=self.cnv.wind_direction(data["wind_direction"]),
                 beaufort=beaufort_data.value,
                 beaufort_description=beaufort_data.description,
                 uv=data["uv"],
-                uv_description=self.calc.uv_description(data["uv"]),
+                uv_description=self.cnv.uv_description(data["uv"]),
                 solar_radiation=data["solar_radiation"],
                 visibility=self.cnv.distance(visibility),
                 lightning_strike_last_epoch=self.cnv.utc_from_timestamp(data["lightning_strike_last_epoch"]),
                 lightning_strike_count=data["lightning_strike_count"],
                 lightning_strike_last_distance=data["lightning_strike_last_distance"],
                 heat_index=self.cnv.temperature(data["heat_index"]),
-                wind_chill=self.cnv.temperature(data["wind_chill"]),
+                wind_chill=self.cnv.temperature(wind_chill),
                 feels_like=self.cnv.temperature(feels_like),
                 dew_point=self.cnv.temperature(data["dew_point"]),
                 trend_temperature=data["trend_temperature"],
-                temperature_trend=self.calc.trend_description(data["trend_temperature"]),
+                temperature_trend=self.cnv.trend_description(data["trend_temperature"]),
                 trend_pressure=data["trend_pressure"],
-                pressure_trend=self.calc.trend_description(data["trend_pressure"]),
+                pressure_trend=self.cnv.trend_description(data["trend_pressure"]),
                 air_pm_10=data["air_pm_10"],
                 air_pm_25=data["air_pm_25"],
                 air_pm_1=data["air_pm_1"],
+                aqi=self.cnv.aqi(data["air_pm_25_havg"]),
                 forecast=data["forecast"],
                 indoor_temperature=self.cnv.temperature(data["indoor_temperature"]),
                 indoor_humidity=data["indoor_humidity"],
+                air_density=self.cnv.air_density(data["air_temperature"], data["station_pressure"]),
+                wet_bulb=self.cnv.temperature(wet_bulb),
                 air_temperature_dmin=self.cnv.temperature(data["air_temperature_dmin"]),
                 air_temperature_dmintime=self.cnv.utc_from_mbtime(data["air_temperature_dmintime"]),
                 air_temperature_dmax=self.cnv.temperature(data["air_temperature_dmax"]),
@@ -169,13 +174,29 @@ class MeteobridgeApiClient:
                 air_temperature_ymintime=self.cnv.utc_from_mbtime(data["air_temperature_ymintime"]),
                 air_temperature_ymax=self.cnv.temperature(data["air_temperature_ymax"]),
                 air_temperature_ymaxtime=self.cnv.utc_from_mbtime(data["air_temperature_ymaxtime"]),
-                is_freezing=self.calc.is_freezing(data["air_temperature"]),
-                is_raining=self.calc.is_raining(data["precip_rate"]),
-                is_lowbat=True if data["is_lowbat"] == 1 else False,
+                temperature_soil_1=self.cnv.temperature(data["temperature_soil_1"]),
+                humidity_soil_1=data["humidity_soil_1"],
+                temperature_soil_2=self.cnv.temperature(data["temperature_soil_2"]),
+                humidity_soil_2=data["humidity_soil_2"],
+                temperature_soil_3=self.cnv.temperature(data["temperature_soil_3"]),
+                humidity_soil_3=data["humidity_soil_3"],
+                temperature_soil_4=self.cnv.temperature(data["temperature_soil_4"]),
+                humidity_soil_4=data["humidity_soil_4"],
+                temperature_leaf_1=self.cnv.temperature(data["temperature_leaf_1"]),
+                humidity_leaf_1=data["humidity_leaf_1"],
+                temperature_leaf_2=self.cnv.temperature(data["temperature_leaf_2"]),
+                humidity_leaf_2=data["humidity_leaf_2"],
+                temperature_leaf_3=self.cnv.temperature(data["temperature_leaf_3"]),
+                humidity_leaf_3=data["humidity_leaf_3"],
+                temperature_leaf_4=self.cnv.temperature(data["temperature_leaf_4"]),
+                humidity_leaf_4=data["humidity_leaf_4"],
+                is_freezing=self.cnv.is_freezing(data["air_temperature"]),
+                is_raining=self.cnv.is_raining(data["precip_rate"]),
             )
 
-            if self.extra_sensors > 0 or self.extra_soil_sensors or self.extra_leaf_sensors > 0:
+            if self.extra_sensors > 0:
                 extra_sensors = await self._get_extra_sensor_values()
+
                 sensor_num = 1
                 while sensor_num < self.extra_sensors + 1:
                     temp_field = f"temperature_extra_{sensor_num}"
@@ -185,20 +206,6 @@ class MeteobridgeApiClient:
                     heat_field = f"heat_index_extra_{sensor_num}"
                     setattr(entity_data, heat_field, self.cnv.temperature(extra_sensors[heat_field]))
                     sensor_num += 1
-                sensor_num = 1
-                while sensor_num < self.extra_soil_sensors + 1:
-                    temp_field = f"temperature_soil_{sensor_num}"
-                    setattr(entity_data, temp_field, self.cnv.temperature(extra_sensors[temp_field]))
-                    hum_field = f"humidity_soil_{sensor_num}"
-                    setattr(entity_data, hum_field, extra_sensors[hum_field])
-                    sensor_num += 1
-                sensor_num = 1
-                while sensor_num < self.extra_leaf_sensors + 1:
-                    temp_field = f"temperature_leaf_{sensor_num}"
-                    setattr(entity_data, temp_field, self.cnv.temperature(extra_sensors[temp_field]))
-                    hum_field = f"humidity_leaf_{sensor_num}"
-                    setattr(entity_data, hum_field, extra_sensors[hum_field])
-                    sensor_num += 1
 
             return entity_data
 
@@ -261,39 +268,11 @@ class MeteobridgeApiClient:
             item_array.append("float")
             sensor_array.append(item_array)
 
-        # Soil Sensors
-        count = 0
-        while count < self.extra_soil_sensors:
-            count += 1
-            item_array = []
-            item_array.append(f"temperature_soil_{count}")
-            item_array.append(f"soil{count}temp-act:None")
-            item_array.append("float")
-            sensor_array.append(item_array)
-            item_array = []
-            item_array.append(f"humidity_soil_{count}")
-            item_array.append(f"soil{count}hum-act.0:None")
-            item_array.append("int")
-            sensor_array.append(item_array)
-
-        # Leaf Sensors
-        count = 0
-        while count < self.extra_leaf_sensors:
-            count += 1
-            item_array = []
-            item_array.append(f"temperature_leaf_{count}")
-            item_array.append(f"leaf{count}temp-act:None")
-            item_array.append("float")
-            sensor_array.append(item_array)
-            item_array = []
-            item_array.append(f"humidity_leaf_{count}")
-            item_array.append(f"leaf{count}hum-act.0:None")
-            item_array.append("int")
-            sensor_array.append(item_array)
-
         data_fields = self._build_endpoint(sensor_array)
         endpoint = f"{self.base_url}{data_fields}"
         result = await self._async_request("get", endpoint)
+        if result is None:
+            return None
         data = await self._process_request_result(result, sensor_array)
         return data
 
@@ -313,17 +292,21 @@ class MeteobridgeApiClient:
         if result is None:
             return None
 
-        items = result.split(";")
-        data = "{"
-        index = 0
-        for data_field in data_fields:
-            enc = "'" if data_field[2] == "str" else ""
-            data += f"'{data_field[0]}': {enc}{items[index]}{enc}, "
-            index += 1
-        data = data[0:-2]
-        data += "}"
+        try:
+            items = result.split(";")
+            data = "{"
+            index = 0
+            for data_field in data_fields:
+                enc = "'" if (data_field[2] == "str" and items[index] != "None") else ""
+                data += f"'{data_field[0]}': {enc}{items[index]}{enc}, "
+                index += 1
+            data = data[0:-2]
+            data += "}"
 
-        return ast.literal_eval(data)
+            return ast.literal_eval(data)
+        except IndexError:
+            _LOGGER.error("No data or faulty data received from Meteobridge Station.")
+            return None
 
     async def _async_request(
         self,
