@@ -26,8 +26,8 @@ FIELDS_OBSERVATION = [
     ["trend_pressure", "thb0seapress-delta10:None", "float"],
     ["air_pm_10", "air0pm-act:None", "float"],
     ["air_pm_25", "air1pm-act:None", "float"],
+    ["air_pm_25_havg", "air1pm-havg:None", "float"],
     ["air_pm_1", "air2pm-act:None", "float"],
-    ["is_lowbat", "th0lowbat-act.0:None", "int"],
     ["forecast", "forecast-text:None", "str"],
     ["indoor_temperature", "thb0temp-act:None", "float"],
     ["indoor_humidity", "thb0hum-act.0:None", "int"],
@@ -43,17 +43,33 @@ FIELDS_OBSERVATION = [
     ["air_temperature_ymintime", "th0temp-ymintime=utc:None", "str"],
     ["air_temperature_ymax", "th0temp-ymax:None", "float"],
     ["air_temperature_ymaxtime", "th0temp-ymaxtime=utc:None", "str"],
+    ["temperature_soil_1", "soil0temp-act:None", "float"],
+    ["humidity_soil_1", "soil0hum-act.0:None", "int"],
+    ["temperature_soil_2", "soil1temp-act:None", "float"],
+    ["humidity_soil_2", "soil1hum-act.0:None", "int"],
+    ["temperature_soil_3", "soil2temp-act:None", "float"],
+    ["humidity_soil_3", "soil2hum-act.0:None", "int"],
+    ["temperature_soil_4", "soil3temp-act:None", "float"],
+    ["humidity_soil_4", "soil3hum-act.0:None", "int"],
+    ["temperature_leaf_1", "leaf0temp-act:None", "float"],
+    ["humidity_leaf_1", "leaf0hum-act.0:None", "int"],
+    ["temperature_leaf_2", "leaf1temp-act:None", "float"],
+    ["humidity_leaf_2", "leaf1hum-act.0:None", "int"],
+    ["temperature_leaf_3", "leaf2temp-act:None", "float"],
+    ["humidity_leaf_3", "leaf2hum-act.0:None", "int"],
+    ["temperature_leaf_4", "leaf3temp-act:None", "float"],
+    ["humidity_leaf_4", "leaf3hum-act.0:None", "int"],
 ]
 
 FIELDS_STATION = [
-    ["mac", "mbsystem-mac:--", "str"],
-    ["swversion", "mbsystem-swversion:--", "float"],
-    ["platform", "mbsystem-platform:--", "str"],
-    ["station", "mbsystem-station:--", "str"],
-    ["timezone", "mbsystem-timezone:--", "str"],
-    ["uptime", "mbsystem-uptime:--", "int"],
+    ["mac", "mbsystem-mac:None", "str"],
+    ["swversion", "mbsystem-swversion:None", "float"],
+    ["platform", "mbsystem-platform:None", "str"],
+    ["station", "mbsystem-station:None", "str"],
+    ["timezone", "mbsystem-timezone:None", "str"],
+    ["uptime", "mbsystem-uptime:0", "int"],
     ["ip", "mbsystem-ip:--", "str"],
-    ["elevation", "mbsystem-altitude:--", "int"],
+    ["elevation", "mbsystem-altitude.0:0", "int"],
 ]
 
 UNIT_TYPE_METRIC = "metric"
